import requests
import base64
from .login import login
from .user import get_username
from .post import post_text, post_text_with_img, upload_pic


def _retry(max_retry=100):
    def fn(func):
        def wrapper(self, *args, **kwargs):
            try:
                return func(self, *args, **kwargs)
            except:
                self.login()
                return func(self, *args, **kwargs)

        return wrapper

    return fn


class Weibo:
    def __init__(self, username, password):
        if not username or not password:
            raise Exception('用户名密码不能为空')

        self.username = username
        self.password = password
        self.session = requests.session()
        self.uuid = 0

    def login(self):
        self.uuid, self.session = login(self.username, self.password)
        self.session.headers.update({
            'Referer': 'https://weibo.com/u/%s/home?topnav=1&wvr=6' % self.uuid,
            'Origin': 'https://weibo.com'
        })

    @_retry()
    def get_username(self):
        return get_username(self.session, self.uuid)

    @_retry()
    def post_text(self, text):
        return post_text(self.session, text)

    @_retry()
    def post_text_with_img(self, text, imgs):
        if imgs is not None and len(imgs) > 0:
            pic_ids = []
            for img in imgs:
                pic_ids.append(self.upload_pic(img))

            return post_text_with_img(self.session, text, '|'.join(pic_ids))
        else:
            return post_text(self.session, text)


     @_retry()
    def post_text_with_b64img(self, text, imgs):
        if imgs is not None and len(imgs) > 0:
            pic_ids = []
            for b64img in imgs:
                pic_ids.append(upload_pic(self.session, b64img))

            return post_text_with_img(self.session, text, '|'.join(pic_ids))
        else:
            return post_text(self.session, text)

    @_retry()
    def upload_pic(self, url):
        b64 = base64.b64encode(requests.get(url).content)
        return upload_pic(self.session, b64)

    def repost(self):
        pass

    def comment(self):
        pass



import pytest
import tests
from time import time
from weibo_web import Weibo

wb = Weibo(tests.username, tests.password)

#
# def test_get_username():
#     assert wb.get_username() == '瞎眼看海贼'
#
#
# def test_post_text():
#     res = wb.post_text('发布的微博lai[悲伤]%s' % int(time() * 1e3))
#
#     assert res['msg'] == ''
#
#
# def test_upload_pic():
#     url = 'https://wx4.sinaimg.cn/orj360/804a43cegy1g7zyxbqimuj21970u0n1a.jpg'
#     pid = wb.upload_pic(url)
#     assert pid != ''


def test_post_text_with_img():
    res = wb.post_text_with_img('发布的图片 %d' % int(time() * 1e3), [
        'https://wx4.sinaimg.cn/orj360/804a43cegy1g7zyxbqimuj21970u0n1a.jpg',
        'https://wx4.sinaimg.cn/orj360/804a43cegy1g7zyxbqimuj21970u0n1a.jpg',
        'https://wx4.sinaimg.cn/orj360/804a43cegy1g7zyxbqimuj21970u0n1a.jpg',
    ])

    assert res['msg'] == ''




import time
import base64
import rsa
import binascii
import requests
import re
import random
try:
    from PIL import Image
except:
    pass
try:
    from urllib.parse import quote_plus
except:
    from urllib import quote_plus

'''
如果没有开启登录保护，不用输入验证码就可以登录
如果开启登录保护，需要输入验证码

'''

# 构造 Request headers
agent = 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_14_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/75.0.3770.90 Safari/537.36'

headers = {
    'User-Agent': agent
}


session = requests.session()

# 访问 初始页面带上 cookie
index_url = "http://weibo.com/login.php"
try:
    session.get(index_url, headers=headers, timeout=2)
except:
    session.get(index_url, headers=headers)


def get_su(username):
    """
    对 email 地址和手机号码 先 javascript 中 encodeURIComponent
    对应 Python 3 中的是 urllib.parse.quote_plus
    然后在 base64 加密后decode
    """
    username_quote = quote_plus(username)
    username_base64 = base64.b64encode(username_quote.encode("utf-8"))
    return username_base64.decode("utf-8")


# 预登陆获得 servertime, nonce, pubkey, rsakv
def get_server_data(su):
    pre_url = "http://login.sina.com.cn/sso/prelogin.php?entry=weibo&callback=sinaSSOController.preloginCallBack&su="
    pre_url = pre_url + su + "&rsakt=mod&checkpin=1&client=ssologin.js(v1.4.18)&_="
    pre_url = pre_url + str(int(time.time() * 1000))
    pre_data_res = session.get(pre_url, headers=headers)

    sever_data = eval(pre_data_res.content.decode("utf-8").replace("sinaSSOController.preloginCallBack", ''))

    return sever_data


# print(sever_data)


def get_password(password, servertime, nonce, pubkey):
    rsaPublickey = int(pubkey, 16)
    key = rsa.PublicKey(rsaPublickey, 65537)  # 创建公钥
    message = str(servertime) + '\t' + str(nonce) + '\n' + str(password)  # 拼接明文js加密文件中得到
    message = message.encode("utf-8")
    passwd = rsa.encrypt(message, key)  # 加密
    passwd = binascii.b2a_hex(passwd)  # 将加密信息转换为16进制。
    return passwd


def get_cha(pcid):
    cha_url = "http://login.sina.com.cn/cgi/pin.php?r="
    cha_url = cha_url + str(int(random.random() * 100000000)) + "&s=0&p="
    cha_url = cha_url + pcid
    cha_page = session.get(cha_url, headers=headers)
    with open("cha.jpg", 'wb') as f:
        f.write(cha_page.content)
        f.close()
    try:
        im = Image.open("cha.jpg")
        im.show()
        im.close()
    except:
        print(u"请到当前目录下，找到验证码后输入")


def login(username, password):
    # su 是加密后的用户名
    su = get_su(username)
    sever_data = get_server_data(su)
    servertime = sever_data["servertime"]
    nonce = sever_data['nonce']
    rsakv = sever_data["rsakv"]
    pubkey = sever_data["pubkey"]
    showpin = sever_data["showpin"]
    password_secret = get_password(password, servertime, nonce, pubkey)

    postdata = {
        'entry': 'weibo',
        'gateway': '1',
        'from': '',
        'savestate': '7',
        'useticket': '1',
        'pagerefer': "http://login.sina.com.cn/sso/logout.php?entry=miniblog&r=http%3A%2F%2Fweibo.com%2Flogout.php%3Fbackurl",
        'vsnf': '1',
        'su': su,
        'service': 'miniblog',
        'servertime': servertime,
        'nonce': nonce,
        'pwencode': 'rsa2',
        'rsakv': rsakv,
        'sp': password_secret,
        'sr': '1366*768',
        'encoding': 'UTF-8',
        'prelt': '115',
        'url': 'http://weibo.com/ajaxlogin.php?framelogin=1&callback=parent.sinaSSOController.feedBackUrlCallBack',
        'returntype': 'META'
        }
    login_url = 'http://login.sina.com.cn/sso/login.php?client=ssologin.js(v1.4.18)'
    if showpin == 0:
        login_page = session.post(login_url, data=postdata, headers=headers)
    else:
        raise Exception('需要验证码...')
        # pcid = sever_data["pcid"]
        # get_cha(pcid)
        # postdata['door'] = input(u"请输入验证码")
        # login_page = session.post(login_url, data=postdata, headers=headers)

    login_loop = login_page.text

    # print(login_loop)
    pa = r'location\.replace\([\'"](.*?)[\'"]\)'
    loop_url = re.findall(pa, login_loop)[0]
    # print(loop_url)
    # 此出还可以加上一个是否登录成功的判断，下次改进的时候写上
    login_index = session.get(loop_url, headers=headers)
    uuid = login_index.text
    uuid_pa = r'"uniqueid":"(.*?)"'
    uuid_res = re.findall(uuid_pa, uuid, re.S)[0]
    return uuid_res, session



import re


def get_username(session, uuid):
    web_weibo_url = "http://weibo.com/%s/profile?topnav=1&wvr=6&is_all=1" % uuid
    weibo_page = session.get(web_weibo_url)

    return re.findall(r"nick\']='([^']+)\';", weibo_page.text)[0]




import time
import re


def post_text(session, text):
    unix = int(time.time() * 1e3)
    url = "https://weibo.com/aj/mblog/add?ajwvr=6&__rnd=%s" % unix

    payload = {
        'location': 'v6_content_home',
        'text': text,
        'appkey': '',
        'style_type': 1,
        'pic_id': '',
        'tid': '',
        'pdetail': '',
        'mid': '',
        'isReEdit': False,
        'gif_ids': '',
        'rank': 0,
        'rankid': '',
        'module': 'stissue',
        'pub_source': 'main_',
        'pub_type': 'dialog',
        'isPri': 0,
        '_t': 0
    }

    r = session.post(url, data=payload)
    r.raise_for_status()
    return r.json()


def post_text_with_img(session, text, pic_id):
    unix = int(time.time() * 1e3)
    url = "https://weibo.com/aj/mblog/add?ajwvr=6&__rnd=%d" % unix

    payload = {
        'location': 'v6_content_home',
        'text': text,
        'appkey': '',
        'style_type': 1,
        'pic_id': pic_id,
        'tid': '',
        'pdetail': '',
        'mid': '',
        'isReEdit': False,
        'gif_ids': '',
        'rank': 0,
        'rankid': '',
        'module': 'stissue',
        'pub_source': 'main_',
        'pub_type': 'dialog',
        'isPri': 0,
        '_t': 0
    }

    r = session.post(url, data=payload)
    r.raise_for_status()
    return r.json()


def upload_pic(session, base64_data):
    url = 'https://picupload.weibo.com/interface/pic_upload.php'
    params = {
        'cb': 'https://weibo.com/aj/static/upimgback.html?_wv=5&callback=STK_ijax_%d' % int(time.time() * 1e3),
        'mime': 'image/jpeg',
        'data': 'base64',
        'url': "",
        'markpos': 1,
        'logo': '',
        'nick': '',
        'marks': 0,
        'app': 'miniblog',
        's': 'rdxt',
        'pri': '',
        'file_source': 1,
    }
    payload = {
        'b64_data': base64_data,
    }
    r = session.post(url, params=params, data=payload, allow_redirects=False)
    # Location: https://weibo.com/aj/static/upimgback.html?_wv=5&callback=STK_ijax_156108350363335&ret=1&pid=804a43cegy1g48k8hemi7j20ig0ag0sv
    location = r.headers.get('location')
    if location is None:
        raise Exception('无法获得pid')
    return re.findall(r'pid=(.*?)$', r.headers.get('location'))[0]
